/*
Introduction:
	This is a pixel movement lib made by VcentG. You can put this in your game and use it for pixel movement.
	Please give credit if you use it. It's not that hard, just put "Used VcentG's Pixel Movement Lib"
	or anything along there to show you used this.

Functions:
	Normal Movement all 4 directions
	Custom Walk
	Custom Walk_To
	Custom Walk_Towards
	Custom Walk_Rand
	Custom Walk_Away
	Custom Step
	Custom Step_To
	Custom Step_Towards
	Custom Step_Rand
	Custom Step_Away

Helpful Hints:
	There is 1 big difference between my Custom Procs and BYOND's Pre-Built ones.
	When using things that you want to function as walk_to like my Walk_To
	This is how they are different:

	-BYOND's walk_to()

	mob/verb/Go()
		blah blah blah
		blah blah blah
		walk_to(REF,TRG,MIN,LAG)


	REF = Atom walking
	TRG = Atom being walked to
	MIN = Number of steps before walking halts
	LAG = Delay between steps


	-My Walk_To()

	mob/verb/Go()
		blah blah blah
		blah blah blah
		REF.Walk_To(TRG,MIN,LAG)

	REF,TRG,MIN,LAG are the same as above, just REF is moved out as a mob proc.

Pixel Movement Lib:
	Last but not least, below is the pixel movement lib, with some side notes to help you understand.
	(Sorry if I don't explain professionally, but maybe this'll help you understand easier)
	Here it is...
*/


atom/movable/animate_movement = NO_STEPS //Leave this =P, without it... you won't want to use this



mob/Login()
	..() //Calls the original Login() which just locates them
	src.icon = 'Character.dmi' //Just sets their icon to the person



mob/Bob //This is Bob, say hi to him
	icon = 'Character.dmi' //Turns out Bob looks just like you!



mob/var
	step = 0 //Which step are you taking? left or right foot!?
	stepping = 0 //Already stepping? Then this will let us know
	pps = 3 //This is PPS (Pixels Per Step)
	target = null //Do you have a target? This is that variable
	walk = 0 //Do you have a preset walk



turf/Grass //Grass turf
	icon = 'Grass.dmi'



turf/Wall //A wall for checking density
	icon = 'Wall.dmi'
	density = 1



client

	North()
		src.mob.PixelMove(0,src.mob.pps)

	South()
		src.mob.PixelMove(0,-src.mob.pps)

	East()
		src.mob.PixelMove(src.mob.pps,0)

	West()
		src.mob.PixelMove(-src.mob.pps,0)

	Northeast()
		src.mob.PixelMove(src.mob.pps,src.mob.pps)

	Southeast()
		src.mob.PixelMove(src.mob.pps,-src.mob.pps)

	Northwest()
		src.mob.PixelMove(-src.mob.pps,src.mob.pps)

	Southwest()
		src.mob.PixelMove(-src.mob.pps,-src.mob.pps)



mob
	verb
		Increase_pps()
			if(src.pps >= 31)
				src << "Your pps can't go above 31!"
				return
			src.pps += 1 //Adds 1 pps
			src << "Your pps is now [src.pps]!" //Message to tell their Pixels Per Step
		Decrease_pps()
			if(src.pps <= 0)
				src << "Your pps can't go below 0!"
				return
			src.pps -= 1 //Takes 1 pps
			src << "Your pps is now [src.pps]!" //Message to tell their Pixels Per Step
		Walk_To_Verb(var/mob/M in oview(6)) //A verb to test Walk_To()
			src.Walk_To(M,1,0) //This is the format you would use to use Walk_To(), and just like walk_to the 0 is not needed.
		Check_Position() //A verb that will tell their location in pixels on the map
			var/xx = src.x //Set it to their x
			var/yy = src.y //Set it to their y
			xx = (xx*32)-32 //Their location with 32 taken off because otherwise you'll go over
			if(src.pixel_x <= -1)
				xx -= src.pixel_x //Takes some pixel_x's off
			else if(src.pixel_x >= 1)
				xx += src.pixel_x //Puts some pixel_x's on
			if(src.pixel_y <= -1)
				yy -= src.pixel_y //Takes some pixel_y's off
			else if(src.pixel_y >= 1)
				yy += src.pixel_y //Puts some pixel_y's on
			src << "Your pixel location is: X ([xx]), Y ([yy])" //Tells you the location in pixels
			src << "Your actual location is: X ([src.x]), Y ([src.y])" //Tells you the location in BYOND's map
	proc
		Walk(var/direction,var/lag)
			src.walk = 0 //Stops other walk_blahs using this
			src.target = null //Don't want them going crazy so this'll deactivate other walks_blahs
			sleep() //Give it a little time to rest, but not too much because we don't like to wait
			if(!direction) return //No shoes? No shirt? No lag? No WALKING
			src.walk = 1 //Ok you have permission to walk
			lag += 3 //Needs a default 3 lag or it'll mess up
			while(src.walk) //Continuous process as long as their walking
				if(direction == NORTH)
					src.PixelMove(0,src.pps)
				else if(direction == SOUTH)
					src.PixelMove(0,-src.pps)
				else if(direction == EAST)
					src.PixelMove(src.pps,0)
				else if(direction == WEST)
					src.PixelMove(-src.pps,0)
				else if(direction == NORTHEAST)
					src.PixelMove(src.pps,src.pps)
				else if(direction == SOUTHEAST)
					src.PixelMove(src.pps,-src.pps)
				else if(direction == NORTHWEST)
					src.PixelMove(-src.pps,src.pps)
				else if(direction == SOUTHWEST)
					src.PixelMove(-src.pps,-src.pps)
				sleep(lag)
		Walk_Away(var/mob/M,var/lag)
			src.target = null
			src.walk = 0
			sleep()
			src.target = M
			lag += 3
			while(M&&src.target==M)
				var/turf/newloc = get_step_away(src,M,1)
				var/direction = get_dir(src.loc,newloc)
				if(direction == NORTH)
					src.PixelMove(0,src.pps)
				else if(direction == SOUTH)
					src.PixelMove(0,-src.pps)
				else if(direction == EAST)
					src.PixelMove(src.pps,0)
				else if(direction == WEST)
					src.PixelMove(-src.pps,0)
				else if(direction == NORTHEAST)
					src.PixelMove(src.pps,src.pps)
				else if(direction == SOUTHEAST)
					src.PixelMove(src.pps,-src.pps)
				else if(direction == NORTHWEST)
					src.PixelMove(-src.pps,src.pps)
				else if(direction == SOUTHWEST)
					src.PixelMove(-src.pps,-src.pps)
				sleep(lag)
		Walk_Rand(var/lag)
			src.walk = 0
			sleep()
			if(!lag) return
			src.walk = 1
			lag += 3
			while(src.walk)
				var/direction = pick(NORTH,EAST,SOUTH,WEST,NORTHEAST,NORTHWEST,SOUTHEAST,SOUTHWEST)
				if(direction == NORTH)
					src.PixelMove(0,src.pps)
				else if(direction == SOUTH)
					src.PixelMove(0,-src.pps)
				else if(direction == EAST)
					src.PixelMove(src.pps,0)
				else if(direction == WEST)
					src.PixelMove(-src.pps,0)
				else if(direction == NORTHEAST)
					src.PixelMove(src.pps,src.pps)
				else if(direction == SOUTHEAST)
					src.PixelMove(src.pps,-src.pps)
				else if(direction == NORTHWEST)
					src.PixelMove(-src.pps,src.pps)
				else if(direction == SOUTHWEST)
					src.PixelMove(-src.pps,-src.pps)
				sleep(lag)
		Walk_To(var/mob/M,var/min,var/lag)
			src.target = null
			sleep()
			src.target = M
			lag += 3
			while(M&&src.target==M)
				for(var/mob/MOBS in oview(min))
					if(MOBS == M)
						return
				var/direction = get_dir(src.loc,M.loc)
				if(direction == NORTH)
					src.PixelMove(0,src.pps)
				else if(direction == SOUTH)
					src.PixelMove(0,-src.pps)
				else if(direction == EAST)
					src.PixelMove(src.pps,0)
				else if(direction == WEST)
					src.PixelMove(-src.pps,0)
				else if(direction == NORTHEAST)
					src.PixelMove(src.pps,src.pps)
				else if(direction == SOUTHEAST)
					src.PixelMove(src.pps,-src.pps)
				else if(direction == NORTHWEST)
					src.PixelMove(-src.pps,src.pps)
				else if(direction == SOUTHWEST)
					src.PixelMove(-src.pps,-src.pps)
				sleep(lag)
		Walk_Towards(var/mob/M,var/lag)
			src.target = null
			sleep()
			src.target = M
			lag += 3
			while(M&&src.target==M)
				var/direction = get_dir(src.loc,M.loc)
				if(direction == NORTH)
					src.PixelMove(0,src.pps)
				else if(direction == SOUTH)
					src.PixelMove(0,-src.pps)
				else if(direction == EAST)
					src.PixelMove(src.pps,0)
				else if(direction == WEST)
					src.PixelMove(-src.pps,0)
				else if(direction == NORTHEAST)
					src.PixelMove(src.pps,src.pps)
				else if(direction == SOUTHEAST)
					src.PixelMove(src.pps,-src.pps)
				else if(direction == NORTHWEST)
					src.PixelMove(-src.pps,src.pps)
				else if(direction == SOUTHWEST)
					src.PixelMove(-src.pps,-src.pps)
				sleep(lag)

		Step(var/direction)
			if(direction == NORTH)
				src.PixelMove(0,src.pps)
			else if(direction == SOUTH)
				src.PixelMove(0,-src.pps)
			else if(direction == EAST)
				src.PixelMove(src.pps,0)
			else if(direction == WEST)
				src.PixelMove(-src.pps,0)
			else if(direction == NORTHEAST)
				src.PixelMove(src.pps,src.pps)
			else if(direction == SOUTHEAST)
				src.PixelMove(src.pps,-src.pps)
			else if(direction == NORTHWEST)
				src.PixelMove(-src.pps,src.pps)
			else if(direction == SOUTHWEST)
				src.PixelMove(-src.pps,-src.pps)
		Step_Away(var/mob/M)
			var/turf/newloc = get_step_away(src,M,1)
			var/direction = get_dir(src.loc,newloc)
			if(direction == NORTH)
				src.PixelMove(0,src.pps)
			else if(direction == SOUTH)
				src.PixelMove(0,-src.pps)
			else if(direction == EAST)
				src.PixelMove(src.pps,0)
			else if(direction == WEST)
				src.PixelMove(-src.pps,0)
			else if(direction == NORTHEAST)
				src.PixelMove(src.pps,src.pps)
			else if(direction == SOUTHEAST)
				src.PixelMove(src.pps,-src.pps)
			else if(direction == NORTHWEST)
				src.PixelMove(-src.pps,src.pps)
			else if(direction == SOUTHWEST)
				src.PixelMove(-src.pps,-src.pps)
		Step_Rand()
			var/direction = pick(NORTH,EAST,SOUTH,WEST,NORTHEAST,NORTHWEST,SOUTHEAST,SOUTHWEST)
			if(direction == NORTH)
				src.PixelMove(0,src.pps)
			else if(direction == SOUTH)
				src.PixelMove(0,-src.pps)
			else if(direction == EAST)
				src.PixelMove(src.pps,0)
			else if(direction == WEST)
				src.PixelMove(-src.pps,0)
			else if(direction == NORTHEAST)
				src.PixelMove(src.pps,src.pps)
			else if(direction == SOUTHEAST)
				src.PixelMove(src.pps,-src.pps)
			else if(direction == NORTHWEST)
				src.PixelMove(-src.pps,src.pps)
			else if(direction == SOUTHWEST)
				src.PixelMove(-src.pps,-src.pps)
		Step_To(var/mob/M,var/min)
			while(M&&src.target==M)
				for(var/mob/MOBS in oview(min))
					if(MOBS == M)
						return
				var/direction = get_dir(src.loc,M.loc)
				if(direction == NORTH)
					src.PixelMove(0,src.pps)
				else if(direction == SOUTH)
					src.PixelMove(0,-src.pps)
				else if(direction == EAST)
					src.PixelMove(src.pps,0)
				else if(direction == WEST)
					src.PixelMove(-src.pps,0)
				else if(direction == NORTHEAST)
					src.PixelMove(src.pps,src.pps)
				else if(direction == SOUTHEAST)
					src.PixelMove(src.pps,-src.pps)
				else if(direction == NORTHWEST)
					src.PixelMove(-src.pps,src.pps)
				else if(direction == SOUTHWEST)
					src.PixelMove(-src.pps,-src.pps)
		Step_Towards(var/mob/M)
			var/direction = get_dir(src.loc,M.loc)
			if(direction == NORTH)
				src.PixelMove(0,src.pps)
			else if(direction == SOUTH)
				src.PixelMove(0,-src.pps)
			else if(direction == EAST)
				src.PixelMove(src.pps,0)
			else if(direction == WEST)
				src.PixelMove(-src.pps,0)
			else if(direction == NORTHEAST)
				src.PixelMove(src.pps,src.pps)
			else if(direction == SOUTHEAST)
				src.PixelMove(src.pps,-src.pps)
			else if(direction == NORTHWEST)
				src.PixelMove(-src.pps,src.pps)
			else if(direction == SOUTHWEST)
				src.PixelMove(-src.pps,-src.pps)


		PixelMove(var/xx,var/yy)

			//Checking and Variable setting

			if(src.stepping) return //Already stepping? Then you can't go again yet! Please wait.
			src.stepping = 1 //Now you're stepping
			src.step += 1 //Adds to your step

			//Animation setting part

			if(src.step >= 3) //Only 2 feet, only 2 step animations, therefore no going over 3
				src.step = 1 //So we set you back to 1

			//Direction Checking/Setting

			if(xx >= 1&&!yy) //If you're going to go east
				src.dir = EAST
			else if(xx <= -1&&!yy) //If you're going to go west
				src.dir = WEST
			else if(yy >= 1&&!xx) //If you're going to go north
				src.dir = NORTH
			else if(yy <= -1&&!xx) //If you're going to go south
				src.dir = SOUTH
			else if(yy >= 1&&xx >= 1) //If you're going to go northeast
				src.dir = NORTHEAST
			else if(yy <= -1&&xx <= -1) //If you're going to go southwest
				src.dir = SOUTHWEST
			else if(yy >= 1&&xx <= -1) //If you're going to go northwest
				src.dir = NORTHWEST
			else if(yy <= -1&&xx >= 1) //If you're going to go southeast
				src.dir = SOUTHEAST

			//Doing the Animation

			if(src.step == 1)
				flick("step 1",src)  //First step animation
			else
				flick("step 2",src)  //Second step animation

			//Setting the x location on the map

			if(src.pixel_x+xx >= 24) //If you're going too far into the next eastern location
				var/turf/T = locate(src.x+1,src.y,src.z) //Check the turf you're moving towards
				if(T&&!T.density) //If it exists (for at map edge) && if its not dense
					src.loc = T//We move you right
					src.pixel_x -= 32 //But you can't tell
				else
					src.stepping = 0
					return
			else if(src.pixel_x+xx <= -24) //If you're going too far into the next western location
				var/turf/T = locate(src.x-1,src.y,src.z) //Check the turf you're moving towards
				if(T&&!T.density) //If it exists (for at map edge) && if its not dense
					src.loc = T//We move you right
					src.pixel_x += 32 //But you can't tell
				else
					src.stepping = 0
					return

			//Setting the y location on the map

			if(src.pixel_y+yy >= 24) //If you're going too far into the next northern location
				var/turf/T = locate(src.x,src.y+1,src.z) //Check the turf you're moving towards
				if(T&&!T.density) //If it exists (for at map edge) && if its not dense
					src.loc = T//We move you right
					src.pixel_y -= 32 //But you can't tell
				else
					src.stepping = 0
					return
			else if(src.pixel_y+yy <= -24) //If you're going too far into the next southern location
				var/turf/T = locate(src.x,src.y-1,src.z) //Check the turf you're moving towards
				if(T&&!T.density) //If it exists (for at map edge) && if its not dense
					src.loc = T//We move you right
					src.pixel_y += 32 //But you can't tell
				else
					src.stepping = 0
					return

			//Setting the pixel x location
			if(src.pixel_x+xx <= -12) //If you're starting to enter the lower area
				var/turf/T = locate(src.x-1,src.y,src.z) //Check the turf you're moving towards
				if(T&&!T.density) //If it exists (for at map edge) && if its not dense
					src.pixel_x += xx
				else if(T&&T.density)
					if(src.pixel_x+xx >= -12)
						src.pixel_x += xx //Moves you
			else if(src.pixel_x+xx >= 12) //If you're starting to enter the upper area
				var/turf/T = locate(src.x+1,src.y,src.z)
				if(T&&!T.density) //If it exists (for at map edge) && if its not dense
					src.pixel_x += xx //Moves you
				else if(T&&T.density)
					if(src.pixel_x+xx <= 12)
						src.pixel_x += xx //Moves you
			else //If you're just moving and not entering anything
				src.pixel_x += xx //Moves you

			//Setting the pixel y location

			if(src.pixel_y+yy <= 0)
				var/turf/T = locate(src.x,src.y-1,src.z)
				if(T&&!T.density) //If it exists (for at map edge) && if its not dense
					src.pixel_y += yy //Moves you
				else if(T&&T.density)
					if(src.pixel_y+yy >= 0)
						src.pixel_y += yy //Moves you
			else if(src.pixel_y+yy >= 30)
				var/turf/T = locate(src.x,src.y+1,src.z)
				if(T&&!T.density) //If it exists (for at map edge) && if its not dense
					src.pixel_y += yy //Moves you
				else if(T&&T.density)
					if(src.pixel_y+yy <= 30)
						src.pixel_y += yy //Moves you
			else //If you're just moving and not entering anything
				src.pixel_y += yy //Moves you

			//Waiting and letting step again

			spawn(4) //Wait 4/10ths a second so the animation is smooth
				src.stepping = 0 //You can now step again
				return //I like to put returns on spawns() this probably isn't needed... But oh well do whatever you want